Package: ggforce
Type: Package
Title: Accelerating 'ggplot2'
Version: 0.5.0
Authors@R: 
    c(person(given = "Thomas Lin",
             family = "Pedersen",
             role = c("cre", "aut"),
             email = "thomasp85@gmail.com",
             comment = c(ORCID = "0000-0002-5147-4711")),
      person("RStudio",
             role = "cph"))
Maintainer: Thomas Lin Pedersen <thomasp85@gmail.com>
Description: The aim of 'ggplot2' is to aid in visual data investigations. This
    focus has led to a lack of facilities for composing specialised plots.
    'ggforce' aims to be a collection of mainly new stats and geoms that fills
    this gap. All additional functionality is aimed to come through the official
    extension system so using 'ggforce' should be a stable experience.
URL: https://ggforce.data-imaginist.com,
        https://github.com/thomasp85/ggforce
BugReports: https://github.com/thomasp85/ggforce/issues
License: MIT + file LICENSE
Encoding: UTF-8
Depends: ggplot2 (>= 3.5.0), R (>= 3.3.0)
Imports: grid, scales, MASS, tweenr (>= 0.1.5), gtable, rlang,
        polyclip, stats, grDevices, tidyselect, withr, utils,
        lifecycle, cli, vctrs, systemfonts
RoxygenNote: 7.3.2
LinkingTo: cpp11
Suggests: sessioninfo, deldir, latex2exp, reshape2, units (>= 0.8.0),
        covr
Collate: 'aaa.R' 'shape.R' 'arc_bar.R' 'arc.R' 'autodensity.R'
        'autohistogram.R' 'autopoint.R' 'bezier.R' 'bspline.R'
        'bspline_closed.R' 'circle.R' 'concaveman.R' 'cpp11.R'
        'diagonal.R' 'diagonal_wide.R' 'ellipse.R' 'errorbar.R'
        'facet_grid_paginate.R' 'facet_matrix.R' 'facet_row.R'
        'facet_stereo.R' 'facet_wrap_paginate.R' 'facet_zoom.R'
        'ggforce-package.R' 'ggproto-classes.R' 'interpolate.R'
        'labeller.R' 'link.R' 'mark_circle.R' 'mark_ellipse.R'
        'mark_hull.R' 'mark_label.R' 'mark_rect.R' 'parallel_sets.R'
        'position-jitternormal.R' 'position_auto.R'
        'position_floatstack.R' 'regon.R' 'scale-depth.R'
        'scale-unit.R' 'sina.R' 'spiro.R' 'themes.R' 'trans.R'
        'trans_linear.R' 'utilities.R' 'voronoi.R' 'zzz.R'
NeedsCompilation: yes
Packaged: 2025-06-18 12:00:16 UTC; thomas
Author: Thomas Lin Pedersen [cre, aut] (ORCID:
    <https://orcid.org/0000-0002-5147-4711>),
  RStudio [cph]
Repository: CRAN
Date/Publication: 2025-06-18 12:40:02 UTC
Built: R 4.6.0; aarch64-apple-darwin23; 2026-04-26 05:29:06 UTC; unix
Archs: ggforce.so.dSYM
