# Generated by using Rcpp::compileAttributes() -> do not edit by hand
# Generator token: 10BE3573-1514-4C36-9D1C-5A225CD40393

splinePath <- function(x, y, degree, knots, detail, type) {
    .Call('_ggforce_splinePath', PACKAGE = 'ggforce', x, y, degree, knots, detail, type)
}

getSplines <- function(x, y, id, detail, type = "clamped") {
    .Call('_ggforce_getSplines', PACKAGE = 'ggforce', x, y, id, detail, type)
}

bezierPath <- function(x, y, detail) {
    .Call('_ggforce_bezierPath', PACKAGE = 'ggforce', x, y, detail)
}

getBeziers <- function(x, y, id, detail) {
    .Call('_ggforce_getBeziers', PACKAGE = 'ggforce', x, y, id, detail)
}

enclose_ellip_points <- function(x, y, id, tol) {
    .Call('_ggforce_enclose_ellip_points', PACKAGE = 'ggforce', x, y, id, tol)
}

enclose_points <- function(x, y, id) {
    .Call('_ggforce_enclose_points', PACKAGE = 'ggforce', x, y, id)
}

points_to_path <- function(pos, path, close) {
    .Call('_ggforce_points_to_path', PACKAGE = 'ggforce', pos, path, close)
}

