# Generated by cpp11: do not edit by hand

splinePath <- function(x, y, degree, knots, detail, type) {
  .Call(`_ggforce_splinePath`, x, y, degree, knots, detail, type)
}

getSplines <- function(x, y, id, detail, type) {
  .Call(`_ggforce_getSplines`, x, y, id, detail, type)
}

bezierPath <- function(x, y, detail) {
  .Call(`_ggforce_bezierPath`, x, y, detail)
}

getBeziers <- function(x, y, id, detail) {
  .Call(`_ggforce_getBeziers`, x, y, id, detail)
}

concaveman_c <- function(p, h, concavity, threshold) {
  .Call(`_ggforce_concaveman_c`, p, h, concavity, threshold)
}

enclose_ellip_points <- function(x, y, id, tol) {
  .Call(`_ggforce_enclose_ellip_points`, x, y, id, tol)
}

enclose_points <- function(x, y, id) {
  .Call(`_ggforce_enclose_points`, x, y, id)
}

points_to_path <- function(pos, path, close) {
  .Call(`_ggforce_points_to_path`, pos, path, close)
}
