% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/shape.R, R/arc_bar.R, R/arc.R, R/autodensity.R,
%   R/autohistogram.R, R/bezier.R, R/bspline.R, R/bspline_closed.R,
%   R/circle.R, R/diagonal.R, R/diagonal_wide.R, R/ellipse.R, R/errorbar.R,
%   R/facet_grid_paginate.R, R/facet_matrix.R, R/facet_row.R, R/facet_stereo.R,
%   R/facet_wrap_paginate.R, R/facet_zoom.R, R/ggproto-classes.R,
%   R/interpolate.R, R/link.R, R/mark_circle.R, R/mark_ellipse.R,
%   R/mark_hull.R, R/mark_rect.R, R/parallel_sets.R, R/position-jitternormal.R,
%   R/position_auto.R, R/position_floatstack.R, R/regon.R, R/sina.R, R/spiro.R,
%   R/voronoi.R
\docType{data}
\name{GeomShape}
\alias{GeomShape}
\alias{StatArcBar}
\alias{StatPie}
\alias{GeomArcBar}
\alias{StatArc}
\alias{GeomArc}
\alias{StatArc2}
\alias{StatArc0}
\alias{GeomArc0}
\alias{StatAutodensity}
\alias{GeomAutoarea}
\alias{StatAutobin}
\alias{GeomAutorect}
\alias{StatBezier}
\alias{StatBezier2}
\alias{StatBezier0}
\alias{GeomBezier0}
\alias{StatBspline}
\alias{StatBspline2}
\alias{GeomBspline0}
\alias{GeomBsplineClosed0}
\alias{StatCircle}
\alias{GeomCircle}
\alias{StatDiagonal}
\alias{StatDiagonal2}
\alias{StatDiagonal0}
\alias{StatDiagonalWide}
\alias{StatEllip}
\alias{StatErr}
\alias{FacetGridPaginate}
\alias{FacetMatrix}
\alias{FacetRow}
\alias{FacetCol}
\alias{FacetStereo}
\alias{FacetWrapPaginate}
\alias{FacetZoom}
\alias{ggforce-extensions}
\alias{GeomPathInterpolate}
\alias{StatLink}
\alias{StatLink2}
\alias{GeomMarkCircle}
\alias{GeomMarkEllipse}
\alias{GeomMarkHull}
\alias{GeomMarkRect}
\alias{StatParallelSets}
\alias{StatParallelSetsAxes}
\alias{GeomParallelSetsAxes}
\alias{PositionJitterNormal}
\alias{PositionAuto}
\alias{PositionFloatstack}
\alias{StatRegon}
\alias{StatSina}
\alias{StatSpiro}
\alias{StatVoronoiTile}
\alias{StatVoronoiSegment}
\alias{StatDelaunayTile}
\alias{StatDelaunaySegment}
\alias{StatDelaunaySegment2}
\alias{StatDelvorSummary}
\title{ggforce extensions to ggplot2}
\description{
ggforce makes heavy use of the ggproto class system to extend the
functionality of ggplot2. In general the actual classes should be of little
interest to users as the standard ggplot2 api of using geom_* and stat_*
functions for building up the plot is encouraged.
}
\keyword{datasets}
